@@ -73,7 +73,7 @@ import org.xml.sax.helpers.XMLReaderFactory;
  * An abstract class that defines some convenience methods for <code>XML</code> parsers.
  *
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
- * @version $Id: AbstractXmlParser.java 785531 2009-06-17 09:47:59Z ltheussl $
+ * @version $Id: AbstractXmlParser.java 807164 2009-08-24 11:59:25Z vsiveton $
  * @since 1.0
  */
 public abstract class AbstractXmlParser
@@ -164,6 +164,9 @@ public abstract class AbstractXmlParser
         {
             throw new ParseException( "Macro execution failed: " + ex.getMessage(), ex );
         }
+
+        setSecondParsing( false );
+        init();
     }
 
     /**
@@ -209,18 +212,6 @@ public abstract class AbstractXmlParser
         return atts;
     }
 
-    /**
-     * Initialize the parser. This is called first by
-     * {@link #parse(java.io.Reader, org.apache.maven.doxia.sink.Sink)} and can be used
-     * to set the parser into a clear state so it can be re-used.
-     *
-     * @since 1.1.1
-     */
-    protected void init()
-    {
-        // default: empty
-    }
-
     /**
      * Parse the model from the XmlPullParser into the given sink.
      *
