@@ -44,7 +44,7 @@ import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
  *
  * @author <a href="mailto:jason@maven.org">Jason van Zyl</a>
  * @author ltheussl
- * @version $Id: XhtmlBaseParser.java 784725 2009-06-15 10:57:11Z vsiveton $
+ * @version $Id: XhtmlBaseParser.java 807164 2009-08-24 11:59:25Z vsiveton $
  * @since 1.1
  */
 public class XhtmlBaseParser
@@ -83,6 +83,8 @@ public class XhtmlBaseParser
     public void parse( Reader source, Sink sink )
         throws ParseException
     {
+        init();
+
         try
         {
             super.parse( source, sink );
@@ -90,6 +92,9 @@ public class XhtmlBaseParser
         finally
         {
             logWarnings();
+
+            setSecondParsing( false );
+            init();
         }
     }
 
@@ -693,6 +698,25 @@ public class XhtmlBaseParser
         return id;
     }
 
+    /** {@inheritDoc} */
+    protected void init()
+    {
+        super.init();
+
+        this.scriptBlock = false;
+        this.isLink = false;
+        this.isAnchor = false;
+        this.orderedListDepth = 0;
+        this.sectionLevel = 0;
+        this.inVerbatim = false;
+        this.inFigure = false;
+        while( this.decoration.getAttributeNames().hasMoreElements() )
+        {
+            this.decoration.removeAttribute( this.decoration.getAttributeNames().nextElement() );
+        }
+        this.warnMessages = null;
+    }
+
     private void handleAEnd( Sink sink )
     {
         if ( isLink )
