@@ -22,8 +22,10 @@ package org.apache.maven.doxia.sink;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.io.Writer;
+import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
@@ -38,13 +40,14 @@ import org.apache.maven.doxia.util.DoxiaUtils;
 import org.apache.maven.doxia.util.HtmlTools;
 
 import org.codehaus.plexus.util.StringUtils;
+import org.codehaus.plexus.util.xml.PrettyPrintXMLWriter;
 
 /**
  * Abstract base xhtml sink implementation.
  *
  * @author Jason van Zyl
  * @author ltheussl
- * @version $Id: XhtmlBaseSink.java 785531 2009-06-17 09:47:59Z ltheussl $
+ * @version $Id: XhtmlBaseSink.java 811611 2009-09-05 09:39:50Z vsiveton $
  * @since 1.1
  */
 public class XhtmlBaseSink
@@ -56,11 +59,7 @@ public class XhtmlBaseSink
     // ----------------------------------------------------------------------
 
     /** The PrintWriter to write the result. */
-    private PrintWriter writer;
-
-    /** The StringWriter to write the result temporary, so we could play with the output and fix XHTML
-     * like DOXIA-177. Calling the method {@link #close()} is needed to perform the changes in the {@link #writer}. */
-    private StringWriter tempWriter;
+    private final PrintWriter writer;
 
     /** Used to collect text events mainly for the head events. */
     private StringBuffer textBuffer = new StringBuffer();
@@ -77,20 +76,30 @@ public class XhtmlBaseSink
     /** An indication on if we're in verbatim mode. */
     private boolean verbatimFlag;
 
-    /** Alignment of table cells. */
-    private int[] cellJustif;
+    /** Stack of alignment int[] of table cells. */
+    private final LinkedList cellJustifStack;
 
-    /** Justification of table cells. */
-    private boolean isCellJustif;
+    /** Stack of justification of table cells. */
+    private final LinkedList isCellJustifStack;
 
-    /** Number of cells in a table row. */
-    private int cellCount;
+    /** Stack of current table cell. */
+    private final LinkedList cellCountStack;
 
     /** Used to style successive table rows differently. */
     private boolean evenTableRow = true;
 
+    /** The stack of StringWriter to write the table result temporary, so we could play with the output DOXIA-177. */
+    private final LinkedList tableContentWriterStack;
+
+    private final LinkedList tableCaptionWriterStack;
+
+    private final LinkedList tableCaptionXMLWriterStack;
+
+    /** The stack of table caption */
+    private final LinkedList tableCaptionStack;
+
     /** used to store attributes passed to table(). */
-    private MutableAttributeSet tableAttributes;
+    protected MutableAttributeSet tableAttributes;
 
     /** Used to distinguish old-style figure handling. */
     private boolean legacyFigure;
@@ -114,7 +123,7 @@ public class XhtmlBaseSink
      * sink.tableRow();
      * </pre>
      * */
-    private boolean tableRows = false;
+    protected boolean tableRows = false;
 
     /** Map of warn messages with a String as key to describe the error type and a Set as value.
      * Using to reduce warn messages. */
@@ -132,7 +141,16 @@ public class XhtmlBaseSink
     public XhtmlBaseSink( Writer out )
     {
         this.writer = new PrintWriter( out );
-        this.tempWriter = new StringWriter();
+
+        this.cellJustifStack = new LinkedList();
+        this.isCellJustifStack = new LinkedList();
+        this.cellCountStack = new LinkedList();
+        this.tableContentWriterStack = new LinkedList();
+        this.tableCaptionWriterStack = new LinkedList();
+        this.tableCaptionXMLWriterStack = new LinkedList();
+        this.tableCaptionStack = new LinkedList();
+
+        init();
     }
 
     // ----------------------------------------------------------------------
@@ -196,8 +214,8 @@ public class XhtmlBaseSink
      */
     protected void setCellJustif( int[] justif )
     {
-        this.cellJustif = justif;
-        this.isCellJustif = true;
+        this.cellJustifStack.addLast( justif );
+        this.isCellJustifStack.addLast( Boolean.TRUE );
     }
 
     /**
@@ -207,7 +225,7 @@ public class XhtmlBaseSink
      */
     protected int[] getCellJustif()
     {
-        return this.cellJustif ;
+        return (int[]) this.cellJustifStack.getLast();
     }
 
     /**
@@ -217,7 +235,7 @@ public class XhtmlBaseSink
      */
     protected void setCellCount( int count )
     {
-        this.cellCount = count;
+        this.cellCountStack.addLast( new Integer( count ) );
     }
 
     /**
@@ -227,21 +245,48 @@ public class XhtmlBaseSink
      */
     protected int getCellCount()
     {
-        return this.cellCount ;
+        return Integer.parseInt( this.cellCountStack.getLast().toString() );
     }
 
     /**
      * Reset all variables.
+     *
+     * @deprecated since 1.1.2, use {@link #init()} instead of.
      */
     protected void resetState()
     {
+        init();
+    }
+
+    /** {@inheritDoc} */
+    protected void init()
+    {
+        super.init();
+
         resetTextBuffer();
-        headFlag = false;
-        verbatimFlag = false;
-        cellJustif = null;
-        isCellJustif = false;
-        cellCount = 0;
-        evenTableRow = true;
+
+        this.headFlag = false;
+        this.verbatimFlag = false;
+        this.evenTableRow = true;
+        this.cellJustifStack.clear();
+        this.isCellJustifStack.clear();
+        this.cellCountStack.clear();
+        this.tableContentWriterStack.clear();
+        this.tableCaptionWriterStack.clear();
+        this.tableCaptionXMLWriterStack.clear();
+        this.tableCaptionStack.clear();
+
+        this.headFlag = false;
+        this.figureCaptionFlag = false;
+        this.paragraphFlag = false;
+        this.verbatimFlag = false;
+        this.evenTableRow = true;
+        this.tableAttributes = null;
+        this.legacyFigure = false;
+        this.legacyFigureCaption = false;
+        this.inFigure = false;
+        this.tableRows = false;
+        this.warnMessages = null;
     }
 
     /**
@@ -844,7 +889,7 @@ public class XhtmlBaseSink
      */
     public void figureGraphics( String name )
     {
-        write( String.valueOf( SPACE ) + Attribute.SRC + EQUAL + QUOTE + name + QUOTE );
+        write( String.valueOf( SPACE ) + Attribute.SRC + EQUAL + QUOTE + escapeHTML( name ) + QUOTE );
     }
 
     /** {@inheritDoc} */
@@ -858,13 +903,19 @@ public class XhtmlBaseSink
             writeStartTag( HtmlMarkup.P, atts );
         }
 
+        MutableAttributeSet filtered = SinkUtils.filterAttributes( attributes, SinkUtils.SINK_IMG_ATTRIBUTES );
+        if ( filtered != null )
+        {
+            filtered.removeAttribute( Attribute.SRC.toString() );
+        }
+
         int count = ( attributes == null ? 1 : attributes.getAttributeCount() + 1 );
 
         MutableAttributeSet atts = new SinkEventAttributeSet( count );
 
-        atts.addAttribute( Attribute.SRC, src );
-        atts.addAttributes( SinkUtils.filterAttributes(
-                attributes, SinkUtils.SINK_IMG_ATTRIBUTES ) );
+        atts.addAttribute( Attribute.SRC, escapeHTML( src ) );
+        atts.addAttributes( filtered );
+
         if ( atts.getAttribute( Attribute.ALT.toString() ) == null )
         {
             atts.addAttribute( Attribute.ALT.toString(), "" );
@@ -1084,6 +1135,7 @@ public class XhtmlBaseSink
     /** {@inheritDoc} */
     public void table( SinkEventAttributes attributes )
     {
+        this.tableContentWriterStack.addLast( new StringWriter() );
         this.tableRows = false;
 
         if ( paragraphFlag )
@@ -1116,49 +1168,41 @@ public class XhtmlBaseSink
 
         writeEndTag( HtmlMarkup.TABLE );
 
-        String content = tempWriter.toString();
-
-        String startTable =
-            new StringBuffer().append( Markup.LESS_THAN ).append( HtmlMarkup.TABLE.toString() ).toString();
-
-        if ( content.lastIndexOf( startTable ) == -1 )
+        if ( !this.cellCountStack.isEmpty() )
         {
-            if ( getLog().isDebugEnabled() )
+            this.cellCountStack.removeLast().toString();
+        }
+
+        if ( this.tableContentWriterStack.isEmpty() )
+        {
+            if ( getLog().isWarnEnabled() )
             {
-                getLog().debug( "table() NOT call firstly" );
+                getLog().warn( "No table content." );
             }
             return;
         }
 
-        content = content.substring( content.lastIndexOf( startTable ) );
+        String tableContent = this.tableContentWriterStack.removeLast().toString();
 
-        String startCaption =
-            new StringBuffer().append( Markup.LESS_THAN ).append( HtmlMarkup.CAPTION.toString() )
-                              .append( Markup.GREATER_THAN ).toString();
-        String endCaption =
-            new StringBuffer().append( Markup.LESS_THAN ).append( Markup.SLASH ).append( HtmlMarkup.CAPTION.toString() )
-                              .append( Markup.GREATER_THAN ).toString();
+        String tableCaption = null;
+        if ( !this.tableCaptionStack.isEmpty() && this.tableCaptionStack.getLast() != null )
+        {
+            tableCaption = this.tableCaptionStack.removeLast().toString();
+        }
 
-        if ( content.indexOf( startCaption ) != -1 && content.indexOf( endCaption ) != -1 )
+        if ( tableCaption != null )
         {
             // DOXIA-177
-            int iStartCaption = content.indexOf( startCaption );
-            int iEndCaption = content.indexOf( endCaption ) + endCaption.length();
+            StringBuffer sb = new StringBuffer();
+            sb.append( tableContent.substring( 0, tableContent.indexOf( Markup.GREATER_THAN ) + 1 ) );
+            sb.append( tableCaption );
+            sb.append( tableContent.substring( tableContent.indexOf( Markup.GREATER_THAN ) + 1 ) );
 
-            String captionTag = content.substring( iStartCaption, iEndCaption );
-            String contentWithoutCaption = StringUtils.replace( content, captionTag, "" );
-
-            String startTr =
-                new StringBuffer().append( Markup.LESS_THAN ).append( HtmlMarkup.TR.toString() ).toString();
-
-            StringBuffer text = new StringBuffer();
-            text.append( contentWithoutCaption.substring( 0, contentWithoutCaption.indexOf( startTr ) ) );
-            text.append( captionTag );
-            text.append( contentWithoutCaption.substring( contentWithoutCaption.indexOf( startTr ) ) );
-
-            String contentWithCaption = tempWriter.toString();
-            tempWriter = new StringWriter();
-            tempWriter.write( StringUtils.replace( contentWithCaption, content, text.toString() ) );
+            write( sb.toString() );
+        }
+        else
+        {
+            write( tableContent );
         }
     }
 
@@ -1173,8 +1217,7 @@ public class XhtmlBaseSink
     {
         this.tableRows = true;
 
-        this.cellJustif = justification;
-        this.isCellJustif = true;
+        setCellJustif( justification );
 
         if ( this.tableAttributes == null )
         {
@@ -1182,35 +1225,41 @@ public class XhtmlBaseSink
         }
 
         MutableAttributeSet att = new SinkEventAttributeSet();
-
-        if ( !tableAttributes.isDefined( Attribute.ALIGN.toString() ) )
+        if ( !this.tableAttributes.isDefined( Attribute.ALIGN.toString() ) )
         {
             att.addAttribute( Attribute.ALIGN, "center" );
         }
 
-        if ( !tableAttributes.isDefined( Attribute.BORDER.toString() ) )
+        if ( !this.tableAttributes.isDefined( Attribute.BORDER.toString() ) )
         {
             att.addAttribute( Attribute.BORDER, ( grid ? "1" : "0" ) );
         }
 
-        if ( !tableAttributes.isDefined( Attribute.CLASS.toString() ) )
+        if ( !this.tableAttributes.isDefined( Attribute.CLASS.toString() ) )
         {
             att.addAttribute( Attribute.CLASS, "bodyTable" );
         }
 
-        att.addAttributes( tableAttributes );
-
-        tableAttributes.removeAttributes( tableAttributes );
+        att.addAttributes( this.tableAttributes );
+        this.tableAttributes.removeAttributes( this.tableAttributes );
 
         writeStartTag( HtmlMarkup.TABLE, att );
+
+        this.cellCountStack.addLast( new Integer( 0 ) );
     }
 
     /** {@inheritDoc} */
     public void tableRows_()
     {
         this.tableRows = false;
-        this.cellJustif = null;
-        this.isCellJustif = false;
+        if ( !this.cellJustifStack.isEmpty() )
+        {
+            this.cellJustifStack.removeLast();
+        }
+        if ( !this.isCellJustifStack.isEmpty() )
+        {
+            this.isCellJustifStack.removeLast();
+        }
 
         this.evenTableRow = true;
     }
@@ -1257,7 +1306,11 @@ public class XhtmlBaseSink
 
         evenTableRow = !evenTableRow;
 
-        cellCount = 0;
+        if ( !this.cellCountStack.isEmpty() )
+        {
+            this.cellCountStack.removeLast();
+            this.cellCountStack.addLast( new Integer( 0 ) );
+        }
     }
 
     /**
@@ -1267,8 +1320,6 @@ public class XhtmlBaseSink
     public void tableRow_()
     {
         writeEndTag( HtmlMarkup.TR );
-
-        cellCount = 0;
     }
 
     /** {@inheritDoc} */
@@ -1321,30 +1372,43 @@ public class XhtmlBaseSink
      */
     private void tableCell( boolean headerRow, MutableAttributeSet attributes )
     {
-        String justif = null;
-
-        if ( cellJustif != null && isCellJustif )
-        {
-            switch ( cellJustif[Math.min( cellCount, cellJustif.length - 1 )] )
-            {
-                case Sink.JUSTIFY_LEFT:
-                    justif = "left";
-                    break;
-                case Sink.JUSTIFY_RIGHT:
-                    justif = "right";
-                    break;
-                case Sink.JUSTIFY_CENTER:
-                default:
-                    justif = "center";
-                    break;
-            }
-        }
-
-
         Tag t = ( headerRow ? HtmlMarkup.TH : HtmlMarkup.TD );
 
         MutableAttributeSet att = new SinkEventAttributeSet();
 
+        String justif = null;
+        if ( attributes == null )
+        {
+            attributes = new SinkEventAttributeSet( 0 );
+        }
+
+        if ( attributes.isDefined( Attribute.ALIGN.toString() ) )
+        {
+            justif = attributes.getAttribute( Attribute.ALIGN.toString() ).toString();
+        }
+
+        if ( !this.cellCountStack.isEmpty() && !this.cellJustifStack.isEmpty() )
+        {
+            int cellCount = Integer.parseInt( this.cellCountStack.getLast().toString() );
+            int[] cellJustif = (int[]) this.cellJustifStack.getLast();
+            if ( justif == null && cellJustif != null && cellJustif.length > 0
+                && this.isCellJustifStack.getLast().equals( Boolean.TRUE ) )
+            {
+                switch ( cellJustif[Math.min( cellCount, cellJustif.length - 1 )] )
+                {
+                    case JUSTIFY_LEFT:
+                        justif = "left";
+                        break;
+                    case JUSTIFY_RIGHT:
+                        justif = "right";
+                        break;
+                    case JUSTIFY_CENTER:
+                    default:
+                        justif = "center";
+                }
+            }
+        }
+
         if ( justif != null )
         {
             att.addAttribute( Attribute.ALIGN, justif );
@@ -1381,9 +1445,11 @@ public class XhtmlBaseSink
 
         writeEndTag( t );
 
-        if ( isCellJustif )
+        if ( !this.isCellJustifStack.isEmpty() && this.isCellJustifStack.getLast().equals( Boolean.TRUE )
+            && !this.cellCountStack.isEmpty() )
         {
-            ++cellCount;
+            int cellCount = Integer.parseInt( this.cellCountStack.removeLast().toString() );
+            this.cellCountStack.addLast( new Integer( ++cellCount ) );
         }
     }
 
@@ -1402,6 +1468,10 @@ public class XhtmlBaseSink
      */
     public void tableCaption( SinkEventAttributes attributes )
     {
+        StringWriter sw = new StringWriter();
+        this.tableCaptionWriterStack.addLast( sw );
+        this.tableCaptionXMLWriterStack.addLast( new PrettyPrintXMLWriter( sw ) );
+
         // TODO: tableCaption should be written before tableRows (DOXIA-177)
         MutableAttributeSet atts = SinkUtils.filterAttributes(
                 attributes, SinkUtils.SINK_SECTION_ATTRIBUTES  );
@@ -1416,6 +1486,12 @@ public class XhtmlBaseSink
     public void tableCaption_()
     {
         writeEndTag( HtmlMarkup.CAPTION );
+
+        if ( !this.tableCaptionXMLWriterStack.isEmpty() && this.tableCaptionXMLWriterStack.getLast() != null )
+        {
+            this.tableCaptionStack.addLast( this.tableCaptionWriterStack.removeLast().toString() );
+            this.tableCaptionXMLWriterStack.removeLast();
+        }
     }
 
     /**
@@ -1821,11 +1897,11 @@ public class XhtmlBaseSink
         {
             if ( tagType == TAG_TYPE_SIMPLE )
             {
-                writeSimpleTag( tag, attributes );
+                writeSimpleTag( tag, escapeAttributeValues( attributes ) );
             }
             else if ( tagType == TAG_TYPE_START )
             {
-                writeStartTag( tag, attributes );
+                writeStartTag( tag, escapeAttributeValues( attributes ) );
             }
             else if ( tagType == TAG_TYPE_END )
             {
@@ -1839,6 +1915,22 @@ public class XhtmlBaseSink
         }
     }
 
+    private SinkEventAttributes escapeAttributeValues( SinkEventAttributes attributes )
+    {
+        SinkEventAttributeSet set = new SinkEventAttributeSet( attributes.getAttributeCount() );
+
+        Enumeration names = attributes.getAttributeNames();
+
+        while ( names.hasMoreElements() )
+        {
+            Object name = names.nextElement();
+
+            set.addAttribute( name, escapeHTML( attributes.getAttribute( name ).toString() ) );
+        }
+
+        return set;
+    }
+
     /** {@inheritDoc} */
     public void flush()
     {
@@ -1848,8 +1940,6 @@ public class XhtmlBaseSink
     /** {@inheritDoc} */
     public void close()
     {
-        writer.write( tempWriter.toString() );
-        tempWriter = new StringWriter();
         writer.close();
 
         if ( getLog().isWarnEnabled() && this.warnMessages != null )
@@ -1870,6 +1960,8 @@ public class XhtmlBaseSink
 
             this.warnMessages = null;
         }
+
+        init();
     }
 
     // ----------------------------------------------------------------------
@@ -1926,7 +2018,63 @@ public class XhtmlBaseSink
     /** {@inheritDoc} */
     protected void write( String text )
     {
-        tempWriter.write( unifyEOLs( text ) );
+        if ( !this.tableCaptionXMLWriterStack.isEmpty() && this.tableCaptionXMLWriterStack.getLast() != null )
+        {
+            ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() ).writeText( unifyEOLs( text ) );
+        }
+        else if ( !this.tableContentWriterStack.isEmpty() && this.tableContentWriterStack.getLast() != null )
+        {
+            ( (StringWriter) this.tableContentWriterStack.getLast() ).write( unifyEOLs( text ) );
+        }
+        else
+        {
+            writer.write( unifyEOLs( text ) );
+        }
+    }
+
+    /** {@inheritDoc} */
+    protected void writeStartTag( Tag t, MutableAttributeSet att, boolean isSimpleTag )
+    {
+        if ( this.tableCaptionXMLWriterStack.isEmpty() )
+        {
+            super.writeStartTag ( t, att, isSimpleTag );
+        }
+        else
+        {
+            String tag = ( getNameSpace() != null ? getNameSpace() + ":" : "" ) + t.toString();
+            ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() ).startElement( tag );
+
+            if ( att != null )
+            {
+                Enumeration names = att.getAttributeNames();
+                while ( names.hasMoreElements() )
+                {
+                    Object key = names.nextElement();
+                    Object value = att.getAttribute( key );
+
+                    ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() )
+                            .addAttribute( key.toString(), value.toString() );
+                }
+            }
+
+            if ( isSimpleTag )
+            {
+                ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() ).endElement();
+            }
+        }
+    }
+
+    /** {@inheritDoc} */
+    protected void writeEndTag( Tag t )
+    {
+        if ( this.tableCaptionXMLWriterStack.isEmpty() )
+        {
+            super.writeEndTag( t );
+        }
+        else
+        {
+            ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() ).endElement();
+        }
     }
 
     /**
