@@ -49,7 +49,7 @@ import java.util.TreeSet;
  * <br/>
  * Based on the <a href="http://www.xmlmind.com/aptconvert.html">APTconvert</a> project.
  *
- * @version $Id: AptParser.java 780266 2009-05-30 13:55:40Z vsiveton $
+ * @version $Id: AptParser.java 812399 2009-09-08 08:57:35Z ltheussl $
  * @since 1.0
  * @plexus.component role="org.apache.maven.doxia.parser.Parser" role-hint="apt"
  */
@@ -187,19 +187,21 @@ public class AptParser
     public void parse( Reader source, Sink sink )
         throws ParseException
     {
+        init();
+
         try
         {
-            try
-            {
-                StringWriter contentWriter = new StringWriter();
-                IOUtil.copy( source, contentWriter );
-                sourceContent = contentWriter.toString();
-            }
-            catch ( IOException e )
-            {
-                throw new AptParseException( "IOException: " + e.getMessage(), e );
-            }
+            StringWriter contentWriter = new StringWriter();
+            IOUtil.copy( source, contentWriter );
+            sourceContent = contentWriter.toString();
+        }
+        catch ( IOException e )
+        {
+            throw new AptParseException( "IOException: " + e.getMessage(), e );
+        }
 
+        try
+        {
             this.source = new AptReaderSource( new StringReader( sourceContent ) );
 
             this.sink = sink;
@@ -218,34 +220,18 @@ public class AptParser
             traverseHead();
 
             traverseBody();
-
-            this.source = null;
-
-            this.sink = null;
         }
         catch ( AptParseException ape )
         {
             // TODO handle column number
             throw new AptParseException( ape.getMessage(), ape, getSourceName(), getSourceLineNumber(), -1 );
         }
-
-        if ( getLog().isWarnEnabled() && this.warnMessages != null && !isSecondParsing() )
+        finally
         {
-            for ( Iterator it = this.warnMessages.entrySet().iterator(); it.hasNext(); )
-            {
-                Map.Entry entry = (Map.Entry) it.next();
+            logWarnings();
 
-                Set set = (Set) entry.getValue();
-
-                for ( Iterator it2 = set.iterator(); it2.hasNext(); )
-                {
-                    String msg = (String) it2.next();
-
-                    getLog().warn( msg );
-                }
-            }
-
-            this.warnMessages = null;
+            setSecondParsing( false );
+            init();
         }
     }
 
@@ -478,7 +464,8 @@ public class AptParser
 
                                 if ( hash.endsWith( ".html" ) && !hash.startsWith( "./" ) )
                                 {
-                                    String msg = "Ambiguous link: '" + hash + "'. If this is a local link, prepend \"./\"!";
+                                    String msg = "Ambiguous link: '" + hash
+                                            + "'. If this is a local link, prepend \"./\"!";
                                     logMessage( "ambiguousLink", msg );
                                 }
 
@@ -707,6 +694,21 @@ public class AptParser
         return replaced.toString();
     }
 
+    /** {@inheritDoc} */
+    protected void init()
+    {
+        super.init();
+
+        this.sourceContent = null;
+        this.sink = null;
+        this.source = null;
+        this.block = null;
+        this.blockFileName = null;
+        this.blockLineNumber = 0;
+        this.line = null;
+        this.warnMessages = null;
+    }
+
     // ----------------------------------------------------------------------
     // Private methods
     // ----------------------------------------------------------------------
@@ -1612,6 +1614,31 @@ public class AptParser
         warnMessages.put( key, set );
     }
 
+    /**
+     * @since 1.1.2
+     */
+    private void logWarnings()
+    {
+        if ( getLog().isWarnEnabled() && this.warnMessages != null && !isSecondParsing() )
+        {
+            for ( Iterator it = this.warnMessages.entrySet().iterator(); it.hasNext(); )
+            {
+                Map.Entry entry = (Map.Entry) it.next();
+
+                Set set = (Set) entry.getValue();
+
+                for ( Iterator it2 = set.iterator(); it2.hasNext(); )
+                {
+                    String msg = (String) it2.next();
+
+                    getLog().warn( msg );
+                }
+            }
+
+            this.warnMessages = null;
+        }
+    }
+
     // -----------------------------------------------------------------------
 
     /** A block of an apt source document. */
@@ -2864,6 +2891,11 @@ public class AptParser
             {
                 String[] param = StringUtils.split( params[i], "=" );
 
+                if ( param.length == 1 )
+                {
+                    throw new AptParseException( "Missing 'key=value' pair for macro parameter: " + params[i] );
+                }
+
                 String key = unescapeForMacro( param[0] );
                 String value = unescapeForMacro( param[1] );
 
