@@ -21,11 +21,13 @@ package org.apache.maven.doxia.module.fo;
 
 import java.io.File;
 import java.io.IOException;
+import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.io.Writer;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.Map;
 import java.util.Set;
 import java.util.Stack;
@@ -44,7 +46,6 @@ import org.apache.maven.doxia.util.HtmlTools;
 
 import org.codehaus.plexus.util.StringUtils;
 import org.codehaus.plexus.util.xml.PrettyPrintXMLWriter;
-import org.codehaus.plexus.util.xml.XMLWriter;
 
 /**
  * A Doxia Sink that produces a FO model. The usage is similar to the following:
@@ -57,7 +58,7 @@ import org.codehaus.plexus.util.xml.XMLWriter;
  * </pre>
  *
  * @author ltheussl
- * @version $Id: FoSink.java 782647 2009-06-08 14:32:26Z ltheussl $
+ * @version $Id: FoSink.java 812141 2009-09-07 13:54:48Z ltheussl $
  * @since 1.1
  */
 public class FoSink
@@ -65,14 +66,10 @@ public class FoSink
     implements FoMarkup
 {
     /** For writing the result. */
-    private final Writer out;
-
-    /** The StringWriter to write the result temporary, so we could play with the output and fix fo.
-     * Calling the method {@link #close()} is needed to perform the changes in the {@link #out}. */
-    private StringWriter tempWriter;
+    private final PrintWriter out;
 
     /** Used to get the current position in numbered lists. */
-    private final Stack listStack = new Stack();
+    private final Stack listStack;
 
     /** Used to get attributes for a given FO element. */
     private final FoConfiguration config;
@@ -86,31 +83,37 @@ public class FoSink
     /** Counts the current subsubsection level. */
     private int subsubsection = 0;
 
-    /** Drawing borders on table cells. */
-    private boolean tableGrid;
-
-    /** Alignment of table cells. */
-    private int[] cellJustif;
-
-    /** Justification of table cells. */
-    private boolean isCellJustif;
-
-    /** Current table cell. */
-    private int cellCount;
-
     /** Verbatim flag. */
     private boolean verbatim;
 
     /** figure flag. */
     private boolean inFigure;
 
-    private String encoding;
+    private final String encoding;
 
-    private String languageId;
+    private final String languageId;
 
-    private StringWriter tableCaptionWriter = null;
+    /** Stack of drawing borders on table cells. */
+    private final LinkedList tableGridStack;
 
-    private XMLWriter tableCaptionXMLWriter = null;
+    /** Stack of alignment int[] of table cells. */
+    private final LinkedList cellJustifStack;
+
+    /** Stack of justification of table cells. */
+    private final LinkedList isCellJustifStack;
+
+    /** Stack of current table cell. */
+    private final LinkedList cellCountStack;
+
+    /** The stack of StringWriter to write the table result temporary, so we could play with the output and fix fo. */
+    private final LinkedList tableContentWriterStack;
+
+    private final LinkedList tableCaptionWriterStack;
+
+    private final LinkedList tableCaptionXMLWriterStack;
+
+    /** The stack of table caption */
+    private final LinkedList tableCaptionStack;
 
     /** Map of warn messages with a String as key to describe the error type and a Set as value.
      * Using to reduce warn messages. */
@@ -136,17 +139,7 @@ public class FoSink
      */
     protected FoSink( Writer writer, String encoding )
     {
-        if ( writer == null )
-        {
-            throw new NullPointerException( "Null writer in FO Sink!" );
-        }
-
-        this.out = writer;
-        this.tempWriter = new StringWriter();
-        this.encoding = encoding;
-        this.config = new FoConfiguration();
-
-        setNameSpace( "fo" );
+        this( writer, encoding, null );
     }
 
     /**
@@ -161,9 +154,27 @@ public class FoSink
      */
     protected FoSink( Writer writer, String encoding, String languageId )
     {
-        this( writer, encoding );
+        if ( writer == null )
+        {
+            throw new NullPointerException( "Null writer in FO Sink!" );
+        }
 
+        this.out = new PrintWriter( writer );
+        this.encoding = encoding;
         this.languageId = languageId;
+        this.config = new FoConfiguration();
+
+        this.listStack = new Stack();
+        this.tableGridStack = new LinkedList();
+        this.cellJustifStack = new LinkedList();
+        this.isCellJustifStack = new LinkedList();
+        this.cellCountStack = new LinkedList();
+        this.tableContentWriterStack = new LinkedList();
+        this.tableCaptionWriterStack = new LinkedList();
+        this.tableCaptionXMLWriterStack = new LinkedList();
+        this.tableCaptionStack = new LinkedList();
+
+        setNameSpace( "fo" );
     }
 
     // TODO add FOP compliance mode?
@@ -187,6 +198,8 @@ public class FoSink
     /** {@inheritDoc} */
     public void head( SinkEventAttributes attributes )
     {
+        init();
+
         startPageSequence( "0", null, null );
     }
 
@@ -600,7 +613,7 @@ public class FoSink
     /** {@inheritDoc} */
     public void numberedList( int numbering, SinkEventAttributes attributes )
     {
-        listStack.push( new NumberedListItem( numbering ) );
+        this.listStack.push( new NumberedListItem( numbering ) );
         writeEOL();
         writeStartTag( LIST_BLOCK_TAG, "list" );
     }
@@ -614,7 +627,7 @@ public class FoSink
     /** {@inheritDoc} */
     public void numberedList_()
     {
-        listStack.pop();
+        this.listStack.pop();
         writeEndTag( LIST_BLOCK_TAG );
         writeEOL();
     }
@@ -622,7 +635,7 @@ public class FoSink
     /** {@inheritDoc} */
     public void numberedListItem( SinkEventAttributes attributes )
     {
-        NumberedListItem current = (NumberedListItem) listStack.peek();
+        NumberedListItem current = (NumberedListItem) this.listStack.peek();
         current.next();
 
         writeStartTag( LIST_ITEM_TAG, "list.item" );
@@ -909,6 +922,7 @@ public class FoSink
         // <fo:table-and-caption> is XSL-FO 1.0 standard but still not implemented in FOP 0.95
         //writeStartTag( TABLE_AND_CAPTION_TAG );
 
+        this.tableContentWriterStack.addLast( new StringWriter() );
         writeStartTag( TABLE_TAG, "table.layout" );
     }
 
@@ -921,49 +935,21 @@ public class FoSink
     /** {@inheritDoc} */
     public void table_()
     {
-        String tableCaption = null;
-        if ( tableCaptionXMLWriter != null )
-        {
-            tableCaption = tableCaptionWriter.toString();
-            tableCaptionXMLWriter = null;
-            tableCaptionWriter = null;
-        }
+        String content = this.tableContentWriterStack.removeLast().toString();
 
-        String content = tempWriter.toString();
-        if ( content.lastIndexOf( "<fo:table " ) != -1 || content.lastIndexOf( "<fo:table>" ) != -1 )
+        StringBuffer sb = new StringBuffer();
+        int cellCount = Integer.parseInt( this.cellCountStack.removeLast().toString() );
+        for ( int i = 0; i < cellCount; i++ )
         {
-            StringBuffer sb = new StringBuffer();
-            // FOP hack to center the table, see
-            // http://xmlgraphics.apache.org/fop/fo.html#fo-center-table-horizon
             sb.append( "<fo:table-column column-width=\"proportional-column-width(1)\"/>" );
             sb.append( EOL );
-
-            int percent = 100 / cellCount;
-            for ( int i = 0; i < cellCount; i++ )
-            {
-                sb.append( "<fo:table-column column-width=\"" + percent + "%\"/>" );
-                sb.append( EOL );
-            }
-
-            sb.append( "<fo:table-column column-width=\"proportional-column-width(1)\"/>" );
-            sb.append( EOL );
-
-            String subContent;
-            if ( content.lastIndexOf( "<fo:table " ) != -1 )
-            {
-                subContent = content.substring( content.lastIndexOf( "<fo:table " ) );
-            }
-            else
-            {
-                subContent = content.substring( content.lastIndexOf( "<fo:table>" ) );
-            }
-            String table = subContent.substring( 0, subContent.indexOf( ">" ) + 1 );
-            String subContentUpdated = StringUtils.replace( subContent, table, table + EOL + sb.toString() );
-
-            tempWriter = new StringWriter();
-            tempWriter.write( StringUtils.replace( content, subContent, subContentUpdated ) );
         }
 
+        int index = content.indexOf( ">" ) + 1;
+        writeln( content.substring( 0, index ) );
+        write( sb.toString() );
+        write( content.substring( index ) );
+
         writeEndTag( TABLE_TAG );
         writeEOL();
 
@@ -973,13 +959,10 @@ public class FoSink
         writeEndTag( BLOCK_TAG );
         writeEOL();
 
-        if ( tableCaption != null )
+        if ( !this.tableCaptionStack.isEmpty() && this.tableCaptionStack.getLast() != null )
         {
-            SinkEventAttributeSet atts = new SinkEventAttributeSet();
-            atts.addAttribute( SinkEventAttributes.ALIGN, "center" );
-
-            paragraph( atts );
-            write( tableCaption );
+            paragraph( SinkEventAttributeSet.CENTER );
+            write( this.tableCaptionStack.removeLast().toString() );
             paragraph_();
         }
     }
@@ -987,9 +970,10 @@ public class FoSink
     /** {@inheritDoc} */
     public void tableRows( int[] justification, boolean grid )
     {
-        this.tableGrid = grid;
-        this.cellJustif = justification;
-        this.isCellJustif = true;
+        this.tableGridStack.addLast( Boolean.valueOf( grid ) );
+        this.cellJustifStack.addLast( justification );
+        this.isCellJustifStack.addLast( Boolean.valueOf( true ) );
+        this.cellCountStack.addLast( new Integer( 0 ) );
         writeEOL();
         writeStartTag( TABLE_BODY_TAG );
     }
@@ -997,8 +981,9 @@ public class FoSink
     /** {@inheritDoc} */
     public void tableRows_()
     {
-        this.cellJustif = null;
-        this.isCellJustif = false;
+        this.tableGridStack.removeLast();
+        this.cellJustifStack.removeLast();
+        this.isCellJustifStack.removeLast();
         writeEndTag( TABLE_BODY_TAG );
         writeEOL();
     }
@@ -1008,7 +993,8 @@ public class FoSink
     {
         // TODO spacer rows
         writeStartTag( TABLE_ROW_TAG, "table.body.row" );
-        this.cellCount = 0;
+        this.cellCountStack.removeLast();
+        this.cellCountStack.addLast( new Integer( 0 ) );
     }
 
     /** {@inheritDoc} */
@@ -1027,7 +1013,7 @@ public class FoSink
     /** {@inheritDoc} */
     public void tableCell( SinkEventAttributes attributes )
     {
-        tableCell( false );
+        tableCell( false, attributes );
     }
 
     /** {@inheritDoc} */
@@ -1040,13 +1026,13 @@ public class FoSink
     public void tableCell( String width )
     {
         // TODO: fop can't handle cell width
-        tableCell( false );
+        tableCell();
     }
 
     /** {@inheritDoc} */
     public void tableHeaderCell( SinkEventAttributes attributes )
     {
-        tableCell( true );
+        tableCell( true, attributes );
     }
 
     /** {@inheritDoc} */
@@ -1059,43 +1045,26 @@ public class FoSink
     public void tableHeaderCell( String width )
     {
         // TODO: fop can't handle cell width
-        tableCell( true );
+        tableHeaderCell();
     }
 
     /**
      * Writes a table cell.
      *
      * @param headerRow true if this is a header cell.
+     * @param attributes the cell attributes, could be null.
      */
-    private void tableCell( boolean headerRow )
+    private void tableCell( boolean headerRow, SinkEventAttributes attributes )
     {
-         String justif = null;
-
-         if ( cellJustif != null && isCellJustif )
-         {
-             switch ( cellJustif[Math.min( cellCount, cellJustif.length - 1 )] )
-             {
-                 case JUSTIFY_LEFT:
-                     justif = "left";
-                     break;
-                 case JUSTIFY_RIGHT:
-                     justif = "right";
-                     break;
-                 case JUSTIFY_CENTER:
-                 default:
-                     justif = "center";
-                     break;
-             }
-         }
-
         MutableAttributeSet cellAtts = headerRow
                  ? config.getAttributeSet( "table.heading.cell" )
                  : config.getAttributeSet( "table.body.cell" );
 
         // the column-number is needed for the hack to center the table, see tableRows.
-        cellAtts.addAttribute( "column-number", String.valueOf( cellCount + 2 ) );
+        int cellCount = Integer.parseInt( this.cellCountStack.getLast().toString() );
+        cellAtts.addAttribute( "column-number", String.valueOf( cellCount + 1 ) );
 
-        if ( tableGrid )
+        if ( this.tableGridStack.getLast().equals( Boolean.TRUE ) )
         {
             cellAtts.addAttributes( config.getAttributeSet( "table.body.cell.grid" ) );
         }
@@ -1104,9 +1073,38 @@ public class FoSink
                  ? config.getAttributeSet( "table.heading.block" )
                  : config.getAttributeSet( "table.body.block" );
 
+        String justif = null;
+        if ( attributes == null )
+        {
+            attributes = new SinkEventAttributeSet( 0 );
+        }
+
+        if ( attributes.isDefined( Attribute.ALIGN.toString() ) )
+        {
+            justif = attributes.getAttribute( Attribute.ALIGN.toString() ).toString();
+        }
+
+        int[] cellJustif = (int[]) this.cellJustifStack.getLast();
+        if ( justif == null && cellJustif != null && cellJustif.length > 0
+            && this.isCellJustifStack.getLast().equals( Boolean.TRUE ) )
+        {
+            switch ( cellJustif[Math.min( cellCount, cellJustif.length - 1 )] )
+            {
+                case JUSTIFY_LEFT:
+                    justif = "left";
+                    break;
+                case JUSTIFY_RIGHT:
+                    justif = "right";
+                    break;
+                case JUSTIFY_CENTER:
+                default:
+                    justif = "center";
+            }
+        }
+
         if ( justif != null )
         {
-           blockAtts.addAttribute( "text-align", justif );
+            blockAtts.addAttribute( "text-align", justif );
         }
 
         writeStartTag( TABLE_CELL_TAG, cellAtts );
@@ -1123,9 +1121,10 @@ public class FoSink
         writeEndTag( TABLE_CELL_TAG );
         writeEOL();
 
-        if ( isCellJustif )
+        if ( this.isCellJustifStack.getLast().equals( Boolean.TRUE ) )
         {
-            ++cellCount;
+            int cellCount = Integer.parseInt( this.cellCountStack.removeLast().toString() );
+            this.cellCountStack.addLast( new Integer( ++cellCount ) );
         }
     }
 
@@ -1138,8 +1137,9 @@ public class FoSink
     /** {@inheritDoc} */
     public void tableCaption( SinkEventAttributes attributes )
     {
-        tableCaptionWriter = new StringWriter();
-        tableCaptionXMLWriter = new PrettyPrintXMLWriter( tableCaptionWriter );
+        StringWriter sw = new StringWriter();
+        this.tableCaptionWriterStack.addLast( sw );
+        this.tableCaptionXMLWriterStack.addLast( new PrettyPrintXMLWriter( sw ) );
 
         // <fo:table-caption> is XSL-FO 1.0 standard but not implemented in FOP 0.95
         //writeStartTag( TABLE_CAPTION_TAG );
@@ -1157,6 +1157,11 @@ public class FoSink
     /** {@inheritDoc} */
     public void tableCaption_()
     {
+        if ( !this.tableCaptionXMLWriterStack.isEmpty() && this.tableCaptionXMLWriterStack.getLast() != null )
+        {
+            this.tableCaptionStack.addLast( this.tableCaptionWriterStack.removeLast().toString() );
+            this.tableCaptionXMLWriterStack.removeLast();
+        }
         // <fo:table-caption> is XSL-FO 1.0 standard but not implemented in FOP 0.95
         //writeEndTag( TABLE_CAPTION_TAG );
     }
@@ -1326,29 +1331,13 @@ public class FoSink
     /** {@inheritDoc} */
     public void flush()
     {
-        try
-        {
-            out.flush();
-        }
-        catch ( IOException e )
-        {
-            getLog().debug( e );
-        }
+        out.flush();
     }
 
     /** {@inheritDoc} */
     public void close()
     {
-        try
-        {
-            out.write( tempWriter.toString() );
-            tempWriter = new StringWriter();
-            out.close();
-        }
-        catch ( IOException e )
-        {
-            getLog().debug( e );
-        }
+        out.close();
 
         if ( getLog().isWarnEnabled() && this.warnMessages != null )
         {
@@ -1368,6 +1357,8 @@ public class FoSink
 
             this.warnMessages = null;
         }
+
+        init();
     }
 
     /**
@@ -1569,13 +1560,17 @@ public class FoSink
      */
     protected void write( String text )
     {
-        if ( tableCaptionXMLWriter == null )
+        if ( !this.tableCaptionXMLWriterStack.isEmpty() && this.tableCaptionXMLWriterStack.getLast() != null )
         {
-            tempWriter.write( unifyEOLs( text ) );
+            ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() ).writeText( unifyEOLs( text ) );
+        }
+        else if ( !this.tableContentWriterStack.isEmpty() && this.tableContentWriterStack.getLast() != null )
+        {
+            ( (StringWriter) this.tableContentWriterStack.getLast() ).write( unifyEOLs( text ) );
         }
         else
         {
-            tableCaptionXMLWriter.writeText( unifyEOLs( text ) );
+            out.write( unifyEOLs( text ) );
         }
     }
 
@@ -1662,14 +1657,14 @@ public class FoSink
     /** {@inheritDoc} */
     protected void writeStartTag( Tag t, MutableAttributeSet att, boolean isSimpleTag )
     {
-        if ( tableCaptionXMLWriter == null )
+        if ( this.tableCaptionXMLWriterStack.isEmpty() )
         {
             super.writeStartTag ( t, att, isSimpleTag );
         }
         else
         {
             String tag = ( getNameSpace() != null ? getNameSpace() + ":" : "" ) + t.toString();
-            tableCaptionXMLWriter.startElement( tag );
+            ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() ).startElement( tag );
 
             if ( att != null )
             {
@@ -1679,13 +1674,14 @@ public class FoSink
                     Object key = names.nextElement();
                     Object value = att.getAttribute( key );
 
-                    tableCaptionXMLWriter.addAttribute( key.toString(), value.toString() );
+                    ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() )
+                            .addAttribute( key.toString(), value.toString() );
                 }
             }
 
             if ( isSimpleTag )
             {
-                tableCaptionXMLWriter.endElement();
+                ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() ).endElement();
             }
         }
     }
@@ -1693,13 +1689,13 @@ public class FoSink
     /** {@inheritDoc} */
     protected void writeEndTag( Tag t )
     {
-        if ( tableCaptionXMLWriter == null )
+        if ( this.tableCaptionXMLWriterStack.isEmpty() )
         {
             super.writeEndTag( t );
         }
         else
         {
-            tableCaptionXMLWriter.endElement();
+            ( (PrettyPrintXMLWriter) this.tableCaptionXMLWriterStack.getLast() ).endElement();
         }
     }
 
@@ -1709,14 +1705,17 @@ public class FoSink
     private static final char DIAMS = 0x2666;
     private static final char EURO = 0x20ac;
     private static final char TRADE = 0x2122;
+    private static final char PRIME = 0x2032;
+    private static final char PPRIME = 0x2033;
 
     private static boolean needsSymbolFont( char c )
     {
         // greek characters and mathematical symbols, except the euro and trade symbols
         // symbols I couldn't get to display in any font:
-        // zwnj (0x200C), zwj (0x200D), lrm (0x200E), rlm (0x200F), oline (0x203E), prime (0x2032),
-        // Prime (0x2033), lceil (0x2038), rceil (0x2039), lfloor (0x203A), rfloor (0x203B)
+        // zwnj (0x200C), zwj (0x200D), lrm (0x200E), rlm (0x200F), oline (0x203E),
+        // lceil (0x2038), rceil (0x2039), lfloor (0x203A), rfloor (0x203B)
         return ( c >= UPPER_ALPHA && c <= PIV )
+                || ( c == PRIME || c == PPRIME )
                 || ( c >= OLINE && c <= DIAMS && c != EURO && c != TRADE );
     }
 
@@ -1806,4 +1805,27 @@ public class FoSink
         set.add( msg );
         warnMessages.put( key, set );
     }
-}
+
+    /** {@inheritDoc} */
+    protected void init()
+    {
+        super.init();
+
+        this.listStack.clear();
+        this.tableGridStack.clear();
+        this.cellJustifStack.clear();
+        this.isCellJustifStack.clear();
+        this.cellCountStack.clear();
+        this.tableContentWriterStack.clear();
+        this.tableCaptionWriterStack.clear();
+        this.tableCaptionXMLWriterStack.clear();
+        this.tableCaptionStack.clear();
+
+        this.section = 0;
+        this.subsection = 0;
+        this.subsubsection = 0;
+        this.verbatim = false;
+        this.inFigure = false;
+        this.warnMessages = null;
+    }
+}
\ No newline at end of file
