@@ -39,8 +39,8 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
 
+import org.apache.maven.doxia.sink.AbstractXmlSink;
 import org.apache.maven.doxia.sink.Sink;
-import org.apache.maven.doxia.sink.SinkAdapter;
 import org.apache.maven.doxia.sink.SinkEventAttributes;
 import org.apache.maven.doxia.util.DoxiaUtils;
 import org.apache.maven.doxia.util.HtmlTools;
@@ -67,10 +67,10 @@ import org.codehaus.plexus.util.xml.XMLWriter;
  * @see <a href="http://www.lowagie.com/iText/tutorial/ch07.html">http://www.lowagie.com/iText/tutorial/ch07.html</a>
  *
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
- * @version $Id: ITextSink.java 782609 2009-06-08 12:52:13Z vsiveton $
+ * @version $Id: ITextSink.java 807173 2009-08-24 12:14:25Z vsiveton $
  */
-public final class ITextSink
-    extends SinkAdapter
+public class ITextSink
+    extends AbstractXmlSink
 {
     /** This is the place where the iText DTD is located. IMPORTANT: this DTD is not uptodate! */
     public static final String DTD = "http://itext.sourceforge.net/itext.dtd";
@@ -94,7 +94,7 @@ public final class ITextSink
     private Writer writer;
 
     /** The XML Writer used */
-    private XMLWriter xmlWriter;
+    private final XMLWriter xmlWriter;
 
     private boolean writeStart;
 
@@ -104,7 +104,7 @@ public final class ITextSink
     /** The font object */
     private ITextFont font;
 
-    private int numberDepth = 0;
+    private int numberDepth = 1;
 
     private int depth = 0;
 
@@ -146,7 +146,7 @@ public final class ITextSink
         this( new PrettyPrintXMLWriter( writer, encoding, null ) );
 
         this.writer = writer;
-        writeStart = true;
+        this.writeStart = true;
     }
 
     /**
@@ -158,11 +158,9 @@ public final class ITextSink
     {
         this.xmlWriter = xmlWriter;
 
-        actionContext = new SinkActionContext();
-        font = new ITextFont();
-        header = new ITextHeader();
+        this.writeStart = false;
 
-        writeStart = false;
+        init();
     }
 
     /**
@@ -212,6 +210,8 @@ public final class ITextSink
 
             this.warnMessages = null;
         }
+
+        init();
     }
 
     /** {@inheritDoc} */
@@ -233,6 +233,8 @@ public final class ITextSink
     /** {@inheritDoc} */
     public void head()
     {
+        init();
+
         actionContext.setAction( SinkActionContext.HEAD );
     }
 
@@ -857,6 +859,8 @@ public final class ITextSink
         writeEndElement(); // ElementTags.CHUNK
 
         actionContext.release();
+
+        lineBreak();
     }
 
     /** {@inheritDoc} */
@@ -870,6 +874,20 @@ public final class ITextSink
         writeAddAttribute( ElementTags.GREEN, font.getFontColorGreen() );
         writeAddAttribute( ElementTags.RED, font.getFontColorRed() );
 
+
+        writeStartElement( ElementTags.CHUNK );
+        writeAddAttribute( ElementTags.FONT, font.getFontName() );
+        writeAddAttribute( ElementTags.SIZE, font.getFontSize() );
+        writeAddAttribute( ElementTags.STYLE, font.getFontStyle() );
+        writeAddAttribute( ElementTags.BLUE, font.getFontColorBlue() );
+        writeAddAttribute( ElementTags.GREEN, font.getFontColorGreen() );
+        writeAddAttribute( ElementTags.RED, font.getFontColorRed() );
+
+        // We need to add a non break space first to display empty string
+        write( "\u00A0" + StringUtils.repeat( " ", 16 ), false, false );
+
+        writeEndElement(); // ElementTags.CHUNK
+
         actionContext.setAction( SinkActionContext.DEFINITION );
     }
 
@@ -1320,6 +1338,10 @@ public final class ITextSink
         font.addUnderlined();
 
         writeStartElement( ElementTags.ANCHOR );
+        if ( StringUtils.isNotEmpty( name )&& name.startsWith( "#" ) && StringUtils.isNotEmpty( header.getTitle() ))
+        {
+            name = "#" + DoxiaUtils.encodeId( header.getTitle(), true ) + "_" + name.substring( 1 );
+        }
         writeAddAttribute( ElementTags.REFERENCE, HtmlTools.escapeHTML( name ) );
         writeAddAttribute( ElementTags.FONT, font.getFontName() );
         writeAddAttribute( ElementTags.SIZE, font.getFontSize() );
@@ -1359,6 +1381,10 @@ public final class ITextSink
             throw new NullPointerException( "Anchor name cannot be null!" );
         }
 
+        if ( StringUtils.isNotEmpty( header.getTitle() ) )
+        {
+            name = header.getTitle() + "_" + name;
+        }
         String id = name;
 
         if ( !DoxiaUtils.isValidId( id ) )
@@ -1565,6 +1591,24 @@ public final class ITextSink
         logMessage( "unknownEvent", msg );
     }
 
+    /** {@inheritDoc} */
+    protected void init()
+    {
+        super.init();
+
+        this.actionContext = new SinkActionContext();
+        this.font = new ITextFont();
+        this.header = new ITextHeader();
+
+        this.numberDepth = 1;
+        this.depth = 0;
+        this.tableCaptionWriter = null;
+        this.tableCaptionXMLWriter = null;
+        this.anchorDefined = false;
+        this.figureDefined = false;
+        this.warnMessages = null;
+    }
+
     /**
      * Convenience method to write a starting element.
      *
@@ -1638,7 +1682,7 @@ public final class ITextSink
      *
      * @param aString
      */
-    private void write( String aString )
+    protected void write( String aString )
     {
         write( aString, false );
     }
