@@ -40,7 +40,7 @@ import org.codehaus.plexus.util.IOUtil;
 /**
  * @author <a href="mailto:jason@maven.org">Jason van Zyl</a>
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse</a>
- * @version $Id: XdocParserTest.java 784385 2009-06-13 12:53:45Z vsiveton $
+ * @version $Id: XdocParserTest.java 806537 2009-08-21 13:06:21Z ltheussl $
  * @since 1.0
  */
 public class XdocParserTest
@@ -48,7 +48,7 @@ public class XdocParserTest
 {
     private XdocParser parser;
 
-    /** @see junit.framework.TestCase#setUp() */
+    /** {@inheritDoc} */
     protected void setUp()
         throws Exception
     {
@@ -189,6 +189,30 @@ public class XdocParserTest
         assertEquals( "body", ( (SinkEventElement) it.next() ).getName() );
         assertEquals( "body_", ( (SinkEventElement) it.next() ).getName() );
         assertFalse( it.hasNext() );
+
+        // DOXIA-359
+        text = "<document>"
+                + "<properties><title>properties title</title></properties>"
+                + "<head><title>head title</title></head>"
+                + "<body></body></document>";
+
+        sink.reset();
+        parser.parse( text, sink );
+
+        it = sink.getEventList().iterator();
+
+        assertEquals( "head", ( (SinkEventElement) it.next() ).getName() );
+        assertEquals( "title", ( (SinkEventElement) it.next() ).getName() );
+
+        SinkEventElement title = (SinkEventElement) it.next();
+        assertEquals( "text", title.getName() );
+        assertEquals( "properties title", title.getArgs()[0] );
+
+        assertEquals( "title_", ( (SinkEventElement) it.next() ).getName() );
+        assertEquals( "head_", ( (SinkEventElement) it.next() ).getName() );
+        assertEquals( "body", ( (SinkEventElement) it.next() ).getName() );
+        assertEquals( "body_", ( (SinkEventElement) it.next() ).getName() );
+        assertFalse( it.hasNext() );
     }
 
     /** @throws Exception  */
